/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * espnet.h
 * Interface to libespnet
 * Copyright (C) 2005 Simon Newton
 */

/** @file espnet/espnet.h */

#ifndef _defined_espnet_h
#define _defined_espnet_h

#include <stdint.h>
// order is important here for osx
#include <sys/types.h>
#include <sys/select.h>

#ifdef __cplusplus
extern "C" {
#endif

// error codes
#define ESPNET_EOK  0
#define ESPNET_ENET -1			// network error
#define ESPNET_EMEM -2			// memory error
#define ESPNET_EARG -3			// argument error
#define ESPNET_ESTATE -4		// state error
#define ESPNET_EACTION -5		// invalid action

typedef enum {
	ESPNET_POLL_REQUEST_FULL = 0x01,
	ESPNET_POLL_REQUEST_ACK = 0x00,
} espnet_poll_request;

// the node types
typedef enum {
	ESPNET_NODE_TYPE_SINGLE_OUT = 0x0001,			// ip to dmx
	ESPNET_NODE_TYPE_SINGLE_IN = 0x0002,			// dmx to ip
	ESPNET_NODE_TYPE_RS232 = 0x0060,
	ESPNET_NODE_TYPE_IO = 0x0061,
	ESPNET_NODE_TYPE_LONWORKS = 0x0100,
} espnet_node_type ;

typedef enum {
	ESPNET_RECV_HANDLER,		/**< Called on reciept of any ArtNet packet */
	ESPNET_SEND_HANDLER,		/**< Called on transmission of any ArtNet packet */
	ESPNET_POLL_HANDLER,		/**< Called on reciept of an ArtPoll packet */
	ESPNET_REPLY_HANDLER,		/**< Called on reciept of an ArtPollReply packet */
	ESPNET_ACK_HANDLER,			/**<  */
	ESPNET_DMX_HANDLER
} espnet_handler_name_t;

/** The local EspNet node */
typedef void *espnet_node ;

extern char *espnet_strerror() ;

// node control functions
extern espnet_node espnet_new(const char *ip, int verbose) ;
extern int espnet_start(espnet_node n) ;
extern int espnet_read(espnet_node n, int timeout) ;
extern int espnet_stop(espnet_node n) ;
int espnet_destroy(espnet_node vn) ;


// handler functions
// these need to be cleaned up to a generic interface
extern int espnet_set_handler(espnet_node vn, espnet_handler_name_t handler, int (*fh)(espnet_node n, void *pp, void * d), void* data) ;
extern int espnet_set_dmx_handler(espnet_node vn, int (*fh)(espnet_node n, uint8_t universe, int length, uint8_t *data,  void *d), void *data );

	
// send functions
extern int espnet_send_poll(espnet_node n, const char *ip, espnet_poll_request type) ;
extern int espnet_send_ack(espnet_node n) ;
extern int espnet_send_poll_reply(espnet_node n) ;
extern int espnet_send_dmx(espnet_node n, uint8_t universe, int16_t length, uint8_t *data) ;


// state changing functions
extern int espnet_set_name(espnet_node vn, const char *name) ;
extern int espnet_set_type(espnet_node vn, espnet_node_type type) ;
extern int espnet_set_uni(espnet_node vn, uint8_t uni) ;

// misc
extern int espnet_get_sd(espnet_node n) ;

#ifdef __cplusplus
}
#endif

#endif
